@@ -16,7 +16,9 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include "gstjpegdec.h"
 #include "gstjpegenc.h"
