@@ -22,7 +22,6 @@
 #endif
 #include "gstmikmod.h"
 
-#include <gst/audio/audio.h>
 #include <stdlib.h>
 
 /* elementfactory information */
@@ -57,52 +56,49 @@ enum {
   ARG_SOFT_SNDFX
 };
 
+MODULE *module;
+MREADER *reader;
+GstPad *srcpad;
+GstClockTime timestamp;
+int need_sync;
 
-static GstPadTemplate*
-mikmod_src_factory (void)
-{
-  static GstPadTemplate *template = NULL;
+static GstStaticPadTemplate mikmod_src_factory =
+GST_STATIC_PAD_TEMPLATE (
+    "src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (
+      "audio/x-raw-int, "
+	"endianness = (int) BYTE_ORDER, "
+	"signed = (boolean) TRUE, "
+	"width = (int) 16, "
+	"depth = (int) 16, "
+	"rate = (int) { 8000, 11025, 22050, 44100 }, "
+	"channels = (int) [ 1, 2 ]; "
+      "audio/x-raw-int, "
+	"signed = (boolean) FALSE, "
+	"width = (int) 8, "
+	"depth = (int) 8, "
+	"rate = (int) { 8000, 11025, 22050, 44100 }, "
+	"channels = (int) [ 1, 2 ]"
+    )
+);
 
-  if (!template) {
-    template = gst_pad_template_new (
-      "src",
-      GST_PAD_SRC,
-      GST_PAD_ALWAYS,
-      gst_caps_new (
-        "mikmod_src",
-        "audio/x-raw-int",
-          GST_AUDIO_INT_PAD_TEMPLATE_PROPS
-      ), NULL);
-  }
-  return template;
-}
-
-
-static GstPadTemplate*
-mikmod_sink_factory (void)
-{
-  static GstPadTemplate *template = NULL;
-
-  if (!template) {
-    template = gst_pad_template_new (
-      "sink",
-      GST_PAD_SINK,
-      GST_PAD_ALWAYS,
-      gst_caps_new (
-        "mikmod_sink",
-        "audio/x-mod",
-        NULL),NULL        
-      );
-  }
-  return template;
-}
+static GstStaticPadTemplate mikmod_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
+    "sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("audio/x-mod")
+);
 
 static void             gst_mikmod_base_init            (gpointer g_class);
 static void		gst_mikmod_class_init		(GstMikModClass *klass);
 static void		gst_mikmod_init			(GstMikMod *filter);
 static void		gst_mikmod_set_property 	(GObject *object, guint id, const GValue *value, GParamSpec *pspec );
 static void		gst_mikmod_get_property		(GObject *object, guint id, GValue *value, GParamSpec *pspec );
-static GstPadLinkReturn	gst_mikmod_srclink		(GstPad *pad, GstCaps *caps);
+static GstPadLinkReturn	gst_mikmod_srclink		(GstPad *pad, const GstCaps *caps);
+static GstCaps *	gst_mikmod_srcfixate		(GstPad *pad, const GstCaps *caps);
 static void             gst_mikmod_loop                 (GstElement *element);
 static gboolean		gst_mikmod_setup 		(GstMikMod *mikmod);
 static GstElementStateReturn  gst_mikmod_change_state 	(GstElement *element);
@@ -137,8 +133,10 @@ gst_mikmod_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class, mikmod_src_factory ());
-  gst_element_class_add_pad_template (element_class, mikmod_sink_factory ());
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mikmod_src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mikmod_sink_factory));
   gst_element_class_set_details (element_class, &mikmod_details);
 }
 
@@ -205,12 +203,15 @@ gst_mikmod_class_init (GstMikModClass *klass)
 static void
 gst_mikmod_init (GstMikMod *filter)
 {  
-  filter->sinkpad = gst_pad_new_from_template(mikmod_sink_factory (),"sink");
-  filter->srcpad = gst_pad_new_from_template(mikmod_src_factory (),"src");
+  filter->sinkpad = gst_pad_new_from_template(
+      gst_static_pad_template_get (&mikmod_sink_factory),"sink");
+  filter->srcpad = gst_pad_new_from_template(
+    gst_static_pad_template_get (&mikmod_src_factory),"src");
 
   gst_element_add_pad(GST_ELEMENT(filter),filter->sinkpad);
   gst_element_add_pad(GST_ELEMENT(filter),filter->srcpad);
   gst_pad_set_link_function (filter->srcpad, gst_mikmod_srclink);
+  gst_pad_set_fixate_function (filter->srcpad, gst_mikmod_srcfixate);
   
   gst_element_set_loop_function (GST_ELEMENT (filter), gst_mikmod_loop);
   
@@ -231,58 +232,49 @@ gst_mikmod_init (GstMikMod *filter)
   filter->modtype     = NULL;
 }
 
-
-static GstPadLinkReturn
-gst_mikmod_negotiate (GstMikMod *mikmod)
+static GstCaps *
+gst_mikmod_srcfixate (GstPad *pad, const GstCaps *caps)
 {
-  gint width, sign;
+  GstCaps *ret;
+  GstStructure *structure;
+  
+  /* FIXME: select est caps here */
+  if (gst_caps_get_size (caps) > 1)
+    return NULL;
 
-  if ( mikmod->_16bit ) {
-    width = 16;
-    sign = TRUE;
-  } else {
-    width = 8;
-    sign = FALSE;
-  }
+  ret = gst_caps_copy (caps);
+  structure = gst_caps_get_structure (ret, 0);
+  
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "channels", 2))
+    return ret;
+  if (gst_caps_structure_fixate_field_nearest_int (structure, "rate", 44100))
+    return ret;
 
-  return gst_pad_try_set_caps (mikmod->srcpad, 
-		          GST_CAPS_NEW (
-			    "mikmod_src",
-			    "audio/x-raw-int",
-			      "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
-			      "signed",      GST_PROPS_BOOLEAN (sign),
-			      "width",       GST_PROPS_INT (width),
-			      "depth",       GST_PROPS_INT (width),
-			      "rate",        GST_PROPS_INT (mikmod->mixfreq),
-			      "channels",    GST_PROPS_INT (mikmod->stereo ? 2 : 1)));
+  gst_caps_free (ret);
+  return NULL;
 }
 
-
 static GstPadLinkReturn
-gst_mikmod_srclink (GstPad *pad, GstCaps *caps)
+gst_mikmod_srclink (GstPad *pad, const GstCaps *caps)
 {
   GstMikMod *filter; 
+  GstStructure *structure;
+  gint depth;
+  gint channels;
 
   filter = GST_MIKMOD (gst_pad_get_parent (pad));
 
-  if (gst_caps_has_property_typed (caps, "depth", GST_PROPS_INT_TYPE)) {
-    gint depth;
-    gst_caps_get_int (caps, "depth", &depth);
-    filter->_16bit = (depth == 16);
-  }
-  if (gst_caps_has_property_typed (caps, "channels", GST_PROPS_INT_TYPE)) {
-    gint channels;
-    gst_caps_get_int (caps, "channels", &channels);
-    filter->stereo = (channels == 2);
-  }
-  if (gst_caps_has_property_typed (caps, "rate", GST_PROPS_INT_TYPE)) {
-    gst_caps_get_int (caps, "rate", &filter->mixfreq);
-  }
+  structure = gst_caps_get_structure (caps, 0);
 
-  return gst_mikmod_negotiate(filter);
+  gst_structure_get_int (structure, "depth", &depth);
+  filter->_16bit = (depth == 16);
+  gst_structure_get_int (structure, "channels", &channels);
+  filter->stereo = (channels == 2);
+  gst_structure_get_int (structure, "rate", &filter->mixfreq);
+  
+  return gst_mikmod_setup (filter) ? GST_PAD_LINK_OK : GST_PAD_LINK_REFUSED;
 }
 
-
 static void
 gst_mikmod_loop (GstElement *element)
 {
@@ -315,13 +307,11 @@ gst_mikmod_loop (GstElement *element)
   }  
   
   if (!GST_PAD_CAPS (mikmod->srcpad)) {
-    if (gst_mikmod_negotiate (mikmod) <= 0) {
-      gst_element_error (GST_ELEMENT (mikmod),
-			 "Failed to negotiate with next element in mikmod");
+    if (GST_PAD_LINK_SUCCESSFUL (gst_pad_renegotiate (mikmod->srcpad))) {
+      GST_ELEMENT_ERROR (mikmod, CORE, NEGOTIATION, (NULL), (NULL));
       return;
     }
   }
-  gst_mikmod_setup( mikmod );
   
   MikMod_RegisterDriver(&drv_gst);
   MikMod_RegisterAllLoaders();
@@ -548,7 +538,7 @@ gst_mikmod_get_property (GObject *object, guint id, GValue *value, GParamSpec *p
 static gboolean
 plugin_init (GstPlugin *plugin)
 {
-  if (!gst_element_register (plugin, "mikmod", GST_RANK_NONE, GST_TYPE_MIKMOD))
+  if (!gst_element_register (plugin, "mikmod", GST_RANK_SECONDARY, GST_TYPE_MIKMOD))
     return FALSE;
 
   return TRUE;
@@ -562,6 +552,5 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   "GPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN)
