@@ -24,7 +24,9 @@
 #endif
 
 #include "tuner.h"
-#include "tunermarshal.h"
+#include "tuner-marshal.h"
+
+#include <string.h>
 
 enum {
   NORM_CHANGED,
@@ -86,14 +88,14 @@ gst_tuner_class_init (GstTunerClass *klass)
 		    NULL, NULL,
 		    g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
 		    GST_TYPE_TUNER_CHANNEL);
-    gst_tuner_signals[NORM_CHANGED] =
+    gst_tuner_signals[FREQUENCY_CHANGED] =
       g_signal_new ("frequency_changed",
 		    GST_TYPE_TUNER, G_SIGNAL_RUN_LAST,
 		    G_STRUCT_OFFSET (GstTunerClass, frequency_changed),
 		    NULL, NULL,
 		    gst_tuner_marshal_VOID__OBJECT_ULONG, G_TYPE_NONE, 2,
 		    GST_TYPE_TUNER_CHANNEL, G_TYPE_ULONG);
-    gst_tuner_signals[NORM_CHANGED] =
+    gst_tuner_signals[SIGNAL_CHANGED] =
       g_signal_new ("signal_changed",
 		    GST_TYPE_TUNER, G_SIGNAL_RUN_LAST,
 		    G_STRUCT_OFFSET (GstTunerClass, signal_changed),
@@ -118,6 +120,16 @@ gst_tuner_class_init (GstTunerClass *klass)
   klass->signal_strength = NULL;
 }
 
+/**
+ * gst_tuner_list_channels:
+ * @tuner: the #GstTuner (a #GstElement) to get the channels from.
+ *
+ * Retrieve a list of channels (e.g. 'composite', 's-video', ...)
+ * from the given tuner object.
+ *
+ * Returns: a list of channels available on this tuner.
+ */
+
 const GList *
 gst_tuner_list_channels (GstTuner *tuner)
 {
@@ -130,6 +142,14 @@ gst_tuner_list_channels (GstTuner *tuner)
   return NULL;
 }
 
+/**
+ * gst_tuner_set_channel:
+ * @tuner: the #GstTuner (a #GstElement) that owns the channel.
+ * @channel: the channel to tune to.
+ *
+ * Tunes the object to the given channel.
+ */
+
 void
 gst_tuner_set_channel (GstTuner        *tuner,
 		       GstTunerChannel *channel)
@@ -141,7 +161,16 @@ gst_tuner_set_channel (GstTuner        *tuner,
   }
 }
 
-const GstTunerChannel *
+/**
+ * gst_Tuner_get_channel:
+ * @tuner: the #GstTuner (a #GstElement) to get the current channel from.
+ *
+ * Retrieve the current channel from the tuner.
+ *
+ * Returns: the current channel of the tuner object.
+ */
+
+GstTunerChannel *
 gst_tuner_get_channel (GstTuner *tuner)
 {
   GstTunerClass *klass = GST_TUNER_GET_CLASS (tuner);
@@ -153,6 +182,17 @@ gst_tuner_get_channel (GstTuner *tuner)
   return NULL;
 }
 
+/**
+ * gst_tuner_get_norms_list:
+ * @tuner: the #GstTuner (*a #GstElement) to get the list of norms from.
+ *
+ * Retrieve a list of available norms on the currently tuned channel
+ * from the given tuner object.
+ *
+ * Returns: A list of norms available on the current channel for this
+ *          tuner object.
+ */
+
 const GList *
 gst_tuner_list_norms (GstTuner *tuner)
 {
@@ -165,6 +205,14 @@ gst_tuner_list_norms (GstTuner *tuner)
   return NULL;
 }
 
+/**
+ * gst_tuner_set_norm:
+ * @tuner: the #GstTuner (a #GstElement) to set the norm on.
+ * @norm: the norm to use for the current channel.
+ *
+ * Changes the video norm on this tuner to the given norm.
+ */
+
 void
 gst_tuner_set_norm (GstTuner     *tuner,
 		    GstTunerNorm *norm)
@@ -176,7 +224,17 @@ gst_tuner_set_norm (GstTuner     *tuner,
   }
 }
 
-const GstTunerNorm *
+/**
+ * gst_tuner_get_norm:
+ * @tuner: the #GstTuner (a #GstElement) to get the current norm from.
+ *
+ * Get the current video norm from the given tuner object for the
+ * currently selected channel.
+ *
+ * Returns: the current norm.
+ */
+
+GstTunerNorm *
 gst_tuner_get_norm (GstTuner *tuner)
 {
   GstTunerClass *klass = GST_TUNER_GET_CLASS (tuner);
@@ -188,6 +246,18 @@ gst_tuner_get_norm (GstTuner *tuner)
   return NULL;
 }
 
+/**
+ * gst_tuner_set_frequency:
+ * @tuner: the #Gsttuner (a #GstElement) that owns the given channel.
+ * @channel: the #GstTunerChannel to set the frequency on.
+ * @frequency: the frequency to tune in to.
+ *
+ * Sets a tuning frequency on the given tuner/channel. Note that this
+ * requires the given channel to be a "tuning" channel, which can be
+ * checked using GST_TUNER_CHANNEL_HAS_FLAG (), with the proper flag
+ * being GST_TUNER_CHANNEL_FREQUENCY.
+ */
+
 void
 gst_tuner_set_frequency (GstTuner        *tuner,
 			 GstTunerChannel *channel,
@@ -203,6 +273,17 @@ gst_tuner_set_frequency (GstTuner        *tuner,
   }
 }
 
+/**
+ * gst_tuner_get_frequency:
+ * @tuner: the #GstTuner (a #GstElement) that owns the given channel.
+ * @channel: the #GstTunerChannel to retrieve the frequency from.
+ *
+ * Retrieve the current frequency from the given channel. The same
+ * applies as for set_frequency (): check the flag.
+ *
+ * Returns: the current frequency, or 0 on error.
+ */
+
 gulong
 gst_tuner_get_frequency (GstTuner        *tuner,
 			 GstTunerChannel *channel)
@@ -219,6 +300,20 @@ gst_tuner_get_frequency (GstTuner        *tuner,
   return 0;
 }
 
+/**
+ * gst_tuner_get_signal_strength:
+ * @tuner: the #GstTuner (a #GstElement) that owns the given channel.
+ * @channel: the #GstTunerChannel to get the signal strength from.
+ *
+ * get the strength of the signal on this channel. Note that this
+ * requires the current channel to be a "tuning" channel, e.g. a
+ * channel on which frequency can be set. This can be checked using
+ * GST_TUNER_CHANNEL_HAS_FLAG (), and the appropriate flag to check
+ * for is GST_TUNER_CHANNEL_FREQUENCY.
+ *
+ * Returns: signal strength, or 0 on error.
+ */
+
 gint
 gst_tuner_signal_strength (GstTuner        *tuner,
 			   GstTunerChannel *channel)
@@ -235,10 +330,49 @@ gst_tuner_signal_strength (GstTuner        *tuner,
   return 0;
 }
 
+GstTunerNorm *
+gst_tuner_find_norm_by_name (GstTuner *tuner,
+			     gchar    *norm)
+{
+  GList *walk;
+
+  g_return_val_if_fail (GST_TUNER (tuner), NULL);
+  g_return_val_if_fail (norm != NULL, NULL);
+
+  walk = (GList *) gst_tuner_list_norms (tuner);
+  while (walk) {
+    if (strcmp (GST_TUNER_NORM (walk->data)->label, norm) == 0)
+      return GST_TUNER_NORM (walk->data);
+    walk = g_list_next (walk);
+  }
+  return NULL;
+}
+
+GstTunerChannel *
+gst_tuner_find_channel_by_name (GstTuner *tuner,
+				gchar    *channel)
+{
+  GList *walk;
+
+  g_return_val_if_fail (GST_TUNER (tuner), NULL);
+  g_return_val_if_fail (channel != NULL, NULL);
+
+  walk = (GList *) gst_tuner_list_channels (tuner);
+  while (walk) {
+    if (strcmp (GST_TUNER_CHANNEL (walk->data)->label, channel) == 0)
+      return GST_TUNER_CHANNEL (walk->data);
+    walk = g_list_next (walk);
+  }
+  return NULL;
+}
+
 void
 gst_tuner_channel_changed (GstTuner        *tuner,
 			   GstTunerChannel *channel)
 {
+  g_return_if_fail (GST_IS_TUNER (tuner));
+  g_return_if_fail (GST_IS_TUNER_CHANNEL (channel));
+
   g_signal_emit (G_OBJECT (tuner),
 		 gst_tuner_signals[CHANNEL_CHANGED], 0,
 		 channel);
@@ -248,6 +382,9 @@ void
 gst_tuner_norm_changed (GstTuner        *tuner,
 			GstTunerNorm    *norm)
 {
+  g_return_if_fail (GST_IS_TUNER (tuner));
+  g_return_if_fail (GST_IS_TUNER_NORM (norm));
+
   g_signal_emit (G_OBJECT (tuner),
 		 gst_tuner_signals[NORM_CHANGED], 0,
 		 norm);
@@ -258,6 +395,9 @@ gst_tuner_frequency_changed (GstTuner        *tuner,
 			     GstTunerChannel *channel,
 			     gulong           frequency)
 {
+  g_return_if_fail (GST_IS_TUNER (tuner));
+  g_return_if_fail (GST_IS_TUNER_CHANNEL (channel));
+
   g_signal_emit (G_OBJECT (tuner),
 		 gst_tuner_signals[FREQUENCY_CHANGED], 0,
 		 channel, frequency);
@@ -272,6 +412,9 @@ gst_tuner_signal_changed (GstTuner        *tuner,
 			  GstTunerChannel *channel,
 			  gint             signal)
 {
+  g_return_if_fail (GST_IS_TUNER (tuner));
+  g_return_if_fail (GST_IS_TUNER_CHANNEL (channel));
+
   g_signal_emit (G_OBJECT (tuner),
 		 gst_tuner_signals[SIGNAL_CHANGED], 0,
 		 channel, signal);
