@@ -20,6 +20,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/* Element-Checklist-Version: 5 */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -51,24 +52,26 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (gst_adder_src_template_factory,
+static GstStaticPadTemplate gst_adder_src_template =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  gst_caps_new ("int_src", "audio/x-raw-int",
-                GST_AUDIO_INT_PAD_TEMPLATE_PROPS),
-  gst_caps_new ("float_src", "audio/x-raw-float",
-                GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS)
+  GST_STATIC_CAPS(
+    GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "
+    GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS
+  )
 );
 
-GST_PAD_TEMPLATE_FACTORY (gst_adder_sink_template_factory,
+static GstStaticPadTemplate gst_adder_sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink%d",
   GST_PAD_SINK,
   GST_PAD_REQUEST,
-  gst_caps_new ("int_sink", "audio/x-raw-int",
-                GST_AUDIO_INT_PAD_TEMPLATE_PROPS),
-  gst_caps_new ("float_sink", "audio/x-raw-float",
-                GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS)
+  GST_STATIC_CAPS(
+    GST_AUDIO_INT_PAD_TEMPLATE_CAPS "; "
+    GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS
+  )
 );
 
 static void 		gst_adder_class_init		(GstAdderClass *klass);
@@ -105,164 +108,69 @@ gst_adder_get_type (void) {
   return adder_type;
 }
 
-static gboolean
-gst_adder_parse_caps (GstAdder *adder, GstCaps *caps)
-{
-  const gchar *mimetype;
-  GstElement *el = GST_ELEMENT (adder);
-
-  mimetype = gst_caps_get_mime (caps);
-
-  if (adder->format == GST_ADDER_FORMAT_UNSET) {
-    /* the caps haven't been set yet at all, so we need to go ahead and set all
-       the relevant values. */
-    if (strcmp (mimetype, "audio/x-raw-int") == 0) {
-      GST_DEBUG ("parse_caps sets adder to format int");
-      adder->format     = GST_ADDER_FORMAT_INT;
-      gst_caps_get_int     (caps, "width",      &adder->width);
-      gst_caps_get_int     (caps, "depth",      &adder->depth);
-      gst_caps_get_int     (caps, "endianness", &adder->endianness);
-      gst_caps_get_boolean (caps, "signed",     &adder->is_signed);
-      gst_caps_get_int     (caps, "channels",   &adder->channels);
-      gst_caps_get_int     (caps, "rate",	&adder->rate);
-    } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
-      GST_DEBUG ("parse_caps sets adder to format float");
-      adder->format     = GST_ADDER_FORMAT_FLOAT;
-      gst_caps_get_int     (caps, "width",     &adder->width);
-      gst_caps_get_int     (caps, "channels",  &adder->channels);
-      gst_caps_get_int     (caps, "rate",      &adder->rate);
-    }
-  } else {
-    /* otherwise, a previously-linked pad has set all the values. we should barf
-       if some of the attempted new values don't match. */
-    if (strcmp (mimetype, "audio/x-raw-int") == 0) {
-      gint width, channels, rate;
-      gboolean is_signed;
-
-      gst_caps_get_int     (caps, "width",     &width);
-      gst_caps_get_int     (caps, "channels",  &channels);
-      gst_caps_get_boolean (caps, "signed",    &is_signed);
-      gst_caps_get_int     (caps, "rate",      &rate);
-
-      /* provide an error message if we can't link */
-      if (adder->format != GST_ADDER_FORMAT_INT) {
-        gst_element_error (el, "can't link a non-int pad to an int adder");
-        return FALSE;
-      }
-      if (adder->channels != channels) {
-        gst_element_error (el,
-                           "can't link %d-channel pad with %d-channel adder",
-                           channels, adder->channels);
-       return FALSE;
-      }
-      if (adder->rate != rate) {
-        gst_element_error (el, "can't link %d Hz pad with %d Hz adder",
-                           rate, adder->rate);
-       return FALSE;
-      }
-      if (adder->width != width) {
-        gst_element_error (el, "can't link %d-bit pad with %d-bit adder",
-                           width, adder->width);
-       return FALSE;
-      }
-      if (adder->is_signed != is_signed) {
-        gst_element_error (el, "can't link %ssigned pad with %ssigned adder",
-                           adder->is_signed ? "" : "un",
-                           is_signed ? "" : "un");
-       return FALSE;
-      }
-    } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
-      gint channels, rate, width;
-
-      gst_caps_get_int (caps, "width",     &width);
-      gst_caps_get_int (caps, "channels",  &channels);
-      gst_caps_get_int (caps, "rate",      &rate);
-
-      if (adder->format != GST_ADDER_FORMAT_FLOAT) {
-        gst_element_error (el, "can't link a non-float pad to a float adder");
-        return FALSE;
-      }
-      if (adder->channels != channels) {
-        gst_element_error (el,
-                           "can't link %d-channel pad with %d-channel adder",
-                           channels, adder->channels);
-        return FALSE;
-      }
-      if (adder->rate != rate) {
-        gst_element_error (el, "can't link %d Hz pad with %d Hz adder",
-                           rate, adder->rate);
-        return FALSE;
-      }
-      if (adder->width != width) {
-        gst_element_error (el, "can't link %d bit float pad with %d bit adder",
-                           width, adder->width);
-        return FALSE;
-      }
-    }
-  }
-  return TRUE;
-}
-
 static GstPadLinkReturn
-gst_adder_link (GstPad *pad, GstCaps *caps)
+gst_adder_link (GstPad *pad, const GstCaps *caps)
 {
   GstAdder *adder;
-  const GList *sinkpads;
-  GList *remove = NULL;
-  GSList *channels;
-  GstPad *p;
+  const char *media_type;
+  const GList *pads;
+  GstStructure *structure;
+  GstPadLinkReturn ret;
+  GstElement *element;
 
   g_return_val_if_fail (caps != NULL, GST_PAD_LINK_REFUSED);
   g_return_val_if_fail (pad  != NULL, GST_PAD_LINK_REFUSED);
 
-  adder = GST_ADDER (GST_PAD_PARENT (pad));
+  element = GST_PAD_PARENT (pad);
+  adder = GST_ADDER (element);
 
-  if (GST_CAPS_IS_FIXED (caps)) {
-    if (!gst_adder_parse_caps (adder, caps))
-      return GST_PAD_LINK_REFUSED;
+  pads = gst_element_get_pad_list (element);
+  while (pads) {
+    GstPad *otherpad = GST_PAD (pads->data);
 
-    if (pad == adder->srcpad || gst_pad_try_set_caps (adder->srcpad, caps) > 0) {
-      sinkpads = gst_element_get_pad_list ((GstElement *) adder);
-      while (sinkpads) {
-        p = (GstPad *) sinkpads->data;
-        if (p != pad && p != adder->srcpad) {
-          if (gst_pad_try_set_caps (p, caps) <= 0) {
-            GST_DEBUG ("caps mismatch; unlinking and removing pad %s:%s "
-		       "(peer %s:%s)",
-                       GST_DEBUG_PAD_NAME (p),
-                       GST_DEBUG_PAD_NAME (GST_PAD_PEER (p)));
-            gst_pad_unlink (GST_PAD (GST_PAD_PEER (p)), p);
-            remove = g_list_prepend (remove, p);
-          }
-        }
-        sinkpads = g_list_next (sinkpads);
+    if (otherpad != pad) {
+      ret = gst_pad_try_set_caps (otherpad, caps);
+      if (GST_PAD_LINK_FAILED (ret)) {
+        return ret;
       }
-      while (remove) {
-        gst_element_remove_pad (GST_ELEMENT (adder),
-                                GST_PAD_CAST (remove->data));
-      restart:
-        channels = adder->input_channels;
-        while (channels) {
-          GstAdderInputChannel *channel;
-	  channel = (GstAdderInputChannel*) channels->data;
-          if (channel->sinkpad == GST_PAD_CAST (remove->data)) {
-            gst_bytestream_destroy (channel->bytestream);
-            adder->input_channels =
-              g_slist_remove_link (adder->input_channels, channels);
-            adder->numsinkpads--;
-            goto restart;
-          }
-          channels = g_slist_next (channels);
-        }
-        remove = g_list_next (remove);
-      }
-      return GST_PAD_LINK_OK;
-    } else {
-      return GST_PAD_LINK_REFUSED;
     }
-  } else {
-    return GST_PAD_LINK_DELAYED;
+    pads = g_list_next (pads);
   }
+
+
+  pads = gst_element_get_pad_list (GST_ELEMENT (adder));
+  while (pads) {
+    GstPad *otherpad = GST_PAD (pads->data);
+
+    if (otherpad != pad) {
+      ret = gst_pad_try_set_caps (otherpad, caps);
+      if (GST_PAD_LINK_FAILED (ret)) {
+        return ret;
+      }
+    }
+    pads = g_list_next (pads);
+  }
+
+  structure = gst_caps_get_structure (caps, 0);
+  media_type = gst_structure_get_name (structure);
+  if (strcmp (media_type, "audio/x-raw-int") == 0) {
+    GST_DEBUG ("parse_caps sets adder to format int");
+    adder->format     = GST_ADDER_FORMAT_INT;
+    gst_structure_get_int     (structure, "width",      &adder->width);
+    gst_structure_get_int     (structure, "depth",      &adder->depth);
+    gst_structure_get_int     (structure, "endianness", &adder->endianness);
+    gst_structure_get_boolean (structure, "signed",     &adder->is_signed);
+    gst_structure_get_int     (structure, "channels",   &adder->channels);
+    gst_structure_get_int     (structure, "rate",	&adder->rate);
+  } else if (strcmp (media_type, "audio/x-raw-float") == 0) {
+    GST_DEBUG ("parse_caps sets adder to format float");
+    adder->format     = GST_ADDER_FORMAT_FLOAT;
+    gst_structure_get_int     (structure, "width",     &adder->width);
+    gst_structure_get_int     (structure, "channels",  &adder->channels);
+    gst_structure_get_int     (structure, "rate",      &adder->rate);
+  }
+  
+  return GST_PAD_LINK_OK;
 }
 
 static void
@@ -275,9 +183,9 @@ gst_adder_class_init (GstAdderClass *klass)
   gstelement_class = (GstElementClass *) klass;
 
   gst_element_class_add_pad_template (gstelement_class,
-    GST_PAD_TEMPLATE_GET (gst_adder_src_template_factory));
+      gst_static_pad_template_get (&gst_adder_src_template));
   gst_element_class_add_pad_template (gstelement_class,
-    GST_PAD_TEMPLATE_GET (gst_adder_sink_template_factory));
+      gst_static_pad_template_get (&gst_adder_sink_template));
   gst_element_class_set_details (gstelement_class, &adder_details);
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
@@ -295,17 +203,17 @@ gst_adder_class_init (GstAdderClass *klass)
 static void 
 gst_adder_init (GstAdder *adder) 
 {
-  adder->srcpad = gst_pad_new_from_template (gst_adder_src_template_factory (),
-                                             "src");
+  adder->srcpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&gst_adder_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (adder), adder->srcpad);
   gst_element_set_loop_function (GST_ELEMENT (adder), gst_adder_loop);
+  gst_pad_set_getcaps_function(adder->srcpad, gst_pad_proxy_getcaps);
   gst_pad_set_link_function (adder->srcpad, gst_adder_link);
 
   adder->format = GST_ADDER_FORMAT_UNSET;
 
   /* keep track of the sinkpads requested */
  
-  adder->bufpool = NULL;
   adder->numsinkpads = 0;
   adder->input_channels = NULL;
 }
@@ -342,6 +250,7 @@ gst_adder_request_new_pad (GstElement *element, GstPadTemplate *templ,
   input->bytestream = gst_bytestream_new (input->sinkpad);
 
   gst_element_add_pad (GST_ELEMENT (adder), input->sinkpad);
+  gst_pad_set_getcaps_function (input->sinkpad, gst_pad_proxy_getcaps);
   gst_pad_set_link_function(input->sinkpad, gst_adder_link);
 
   /* add the input_channel to the list of input channels */
@@ -398,19 +307,13 @@ gst_adder_loop (GstElement *element)
 
   adder = GST_ADDER (element);
 
-  if (adder->bufpool == NULL) {
-    adder->bufpool = gst_pad_get_bufferpool (adder->srcpad);
-    if (adder->bufpool == NULL) {
-      adder->bufpool = gst_buffer_pool_get_default (GST_ADDER_BUFFER_SIZE,
-	                                            GST_ADDER_NUM_BUFFERS);
-    }
-  }
-
   /* get new output buffer */
-  buf_out = gst_buffer_new_from_pool (adder->bufpool, 0, 0);
+  /* FIXME the 1024 is arbitrary */
+  buf_out = gst_buffer_new_and_alloc (1024);
 
   if (buf_out == NULL) {
-    gst_element_error (GST_ELEMENT (adder), "could not get new output buffer");
+    GST_ELEMENT_ERROR (adder, CORE, TOO_LAZY, (NULL),
+                       ("could not get new output buffer"));
     return;
   }
 
@@ -498,9 +401,9 @@ gst_adder_loop (GstElement *element)
           for (i = 0; i < GST_BUFFER_SIZE (buf_out); i++)
             out[i] = CLAMP(out[i] + in[i], 0x80, 0x7f);
         } else {
-          gst_element_error (GST_ELEMENT (adder),
-		     "invalid width (%u) for integer audio in gstadder",
-		     adder->width);
+          GST_ELEMENT_ERROR (adder, STREAM, FORMAT, (NULL),
+		     ("invalid width (%u) for integer audio in gstadder",
+		     adder->width));
 	  return;
         }
       } else if (adder->format == GST_ADDER_FORMAT_FLOAT) {
@@ -515,15 +418,15 @@ gst_adder_loop (GstElement *element)
           for (i = 0; i < GST_BUFFER_SIZE (buf_out) / sizeof (gfloat); i++)
             out[i] = CLAMP(out[i] + in[i], -1.0, 1.0);
         } else {
-          gst_element_error (GST_ELEMENT (adder),
-                     "invalid width (%u) for float audio in gstadder",
-                     adder->width);
+          GST_ELEMENT_ERROR (adder, STREAM, FORMAT, (NULL),
+                     ("invalid width (%u) for float audio in gstadder",
+                     adder->width));
           return;
         }
       } else {
-        gst_element_error (GST_ELEMENT (adder),
-	           "invalid audio format (%d) in gstadder",
-	           adder->format);
+        GST_ELEMENT_ERROR (adder, STREAM, FORMAT, (NULL),
+	           ("invalid audio format (%d) in gstadder",
+	           adder->format));
 	return;
       }
 
@@ -533,22 +436,6 @@ gst_adder_loop (GstElement *element)
     }
   }
 
-  if (adder->format == GST_ADDER_FORMAT_UNSET) {
-    GstCaps *caps =
-      gst_caps_new ("default_adder_caps",
-                    "audio/x-raw-int",
-                    GST_AUDIO_INT_PAD_TEMPLATE_PROPS);
-
-    if (gst_pad_try_set_caps (adder->srcpad, caps) < 0) {
-      gst_element_error (GST_ELEMENT (adder),
-                         "Couldn't set the default caps, "
-                         "use link_filtered instead");
-      return;
-    }
-
-    gst_adder_parse_caps (adder, caps);
-  }
-
   GST_BUFFER_TIMESTAMP (buf_out) = adder->timestamp;
   if (adder->format == GST_ADDER_FORMAT_FLOAT)
     adder->offset += GST_BUFFER_SIZE (buf_out) / sizeof (gfloat) / adder->channels;
@@ -580,10 +467,6 @@ gst_adder_change_state (GstElement *element)
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
-      if (adder->bufpool) {
-	gst_buffer_pool_unref (adder->bufpool);
-	adder->bufpool = NULL;
-      }
       break;
     case GST_STATE_PAUSED_TO_READY:
       break;
@@ -622,7 +505,6 @@ GST_PLUGIN_DEFINE(
   plugin_init,
   VERSION,
   "LGPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
