@@ -156,22 +156,22 @@ gst_ebml_read_element_id (GstEbmlRead *ebml,
   gint len_mask = 0x80, read = 1, n = 1;
   guint32 total;
 
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
     GstEvent *event = NULL;
     guint32 remaining;
 
     /* Here, we might encounter EOS */
     gst_bytestream_get_status (ebml->bs, &remaining, &event);
-    if (event && GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+    if (event) {
       gst_pad_event_default (ebml->sinkpad, event);
     } else {
       guint64 pos = gst_bytestream_tell (ebml->bs);
       gst_event_unref (event);
-      gst_element_error (GST_ELEMENT (ebml),
-			 "Read error at position %llu (0x%llx)",
-			 pos, pos);
+      GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
+			 ("Read error at position %llu (0x%llx)",
+			 pos, pos));
+      return -1;
     }
-    return -1;
   }
   total = data[0];
   while (read <= 4 && !(total & len_mask)) {
@@ -180,17 +180,16 @@ gst_ebml_read_element_id (GstEbmlRead *ebml,
   }
   if (read > 4) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Invalid EBML ID size tag (0x%x) at position %llu (0x%llx)",
-		       data[0], pos, pos);
+    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
+		       ("Invalid EBML ID size tag (0x%x) at position %llu (0x%llx)",
+		       data[0], pos, pos));
     return -1;
   }
 
   if (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Read error at position %llu (0x%llx)",
-		       pos, pos);
+    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
+		       ("Read error at position %llu (0x%llx)", pos, pos));
     return -1;
   }
   while (n < read)
@@ -217,12 +216,11 @@ gst_ebml_read_element_length (GstEbmlRead *ebml,
   guint8 *data;
   gint len_mask = 0x80, read = 1, n = 1, num_ffs = 0;
   guint64 total;
-                                                                                
+
   if (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Read error at position %llu (0x%llx)",
-		       pos, pos);
+    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
+		       ("Read error at position %llu (0x%llx)", pos, pos));
     return -1;
   }
   total = data[0];
@@ -232,9 +230,9 @@ gst_ebml_read_element_length (GstEbmlRead *ebml,
   }
   if (read > 8) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Invalid EBML length size tag (0x%x) at position %llu (0x%llx)",
-		       data[0], pos, pos);
+    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
+		       ("Invalid EBML length size tag (0x%x) at position %llu (0x%llx)",
+		       data[0], pos, pos));
     return -1;
   }
 
@@ -242,9 +240,8 @@ gst_ebml_read_element_length (GstEbmlRead *ebml,
     num_ffs++;
   if (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Read error at position %llu (0x%llx)",
-		       pos, pos);
+    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
+		       ("Read error at position %llu (0x%llx)", pos, pos));
     return -1;
   }
   while (n < read) {
@@ -275,9 +272,8 @@ gst_ebml_read_element_data (GstEbmlRead *ebml,
 
   if (gst_bytestream_peek (ebml->bs, &buf, length) != length) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Read error at position %llu (0x%llx)",
-		       pos, pos);
+    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
+		       ("Read error at position %llu (0x%llx)", pos, pos));
     if (buf)
       gst_buffer_unref (buf);
     return NULL;
@@ -317,7 +313,7 @@ gst_ebml_read_seek (GstEbmlRead *ebml,
 		    guint64      offset)
 {
   guint32 remaining;
-  GstEvent *event;
+  GstEvent *event = NULL;
   guchar *data;
 
   /* first, flush remaining buffers */
@@ -331,25 +327,28 @@ gst_ebml_read_seek (GstEbmlRead *ebml,
 
   /* now seek */
   if (!gst_bytestream_seek (ebml->bs, offset, GST_SEEK_METHOD_SET)) {
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Seek to position %llu (0x%llx) failed",
-		       offset, offset);
+    GST_ELEMENT_ERROR (ebml, RESOURCE, SEEK, (NULL),
+		       ("Seek to position %llu (0x%llx) failed", offset, offset));
     return NULL;
   }
 
-  /* and now, peek a new byte. This will fail because there's a
-   * pending event. Then, take the event and return it. */
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, 1))
-    g_warning ("Unexpected data after seek");
+  while (!event) {
+    /* and now, peek a new byte. This will fail because there's a
+     * pending event. Then, take the event and return it. */
+    if (gst_bytestream_peek_bytes (ebml->bs, &data, 1)) {
+      GST_WARNING ("Unexpected data after seek - this means seek failed");
+      break;
+    }
 
-  /* get the discont event and return */
-  gst_bytestream_get_status (ebml->bs, &remaining, &event);
-  if (!event || GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
-    gst_element_error (GST_ELEMENT (ebml),
-		       "No discontinuity event after seek");
-    if (event)
-      gst_event_unref (event);
-    return NULL;
+    /* get the discont event and return */
+    gst_bytestream_get_status (ebml->bs, &remaining, &event);
+    if (!event) {
+      GST_WARNING ("No discontinuity event after seek - seek failed");
+      break;
+    } else if (GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
+      gst_pad_event_default (ebml->sinkpad, event);
+      event = NULL;
+    }
   }
 
   return event;
@@ -436,9 +435,9 @@ gst_ebml_read_uint (GstEbmlRead *ebml,
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
   if (size < 1 || size > 8) {
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Invalid integer element size %d at position %llu (0x%llu)",
-		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf));
+    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
+		       ("Invalid integer element size %d at position %llu (0x%llu)",
+		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
     gst_buffer_unref (buf);
     return FALSE;
   }
@@ -464,7 +463,7 @@ gst_ebml_read_sint (GstEbmlRead *ebml,
 {
   GstBuffer *buf;
   guint8 *data;
-  guint size;
+  guint size, negative = 0, n = 0;
 
   if (!gst_ebml_read_buffer (ebml, id, &buf))
     return FALSE;
@@ -472,20 +471,25 @@ gst_ebml_read_sint (GstEbmlRead *ebml,
   data = GST_BUFFER_DATA (buf);
   size = GST_BUFFER_SIZE (buf);
   if (size < 1 || size > 8) {
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Invalid integer element size %d at position %llu (0x%llx)",
-		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf));
+    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
+		       ("Invalid integer element size %d at position %llu (0x%llx)",
+		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
     gst_buffer_unref (buf);
     return FALSE;
   }
+  if (data[0] & 0x80) {
+    negative = 1;
+    data[0] &= ~0x80;
+  }
   *num = 0;
-  while (size > 0) {
-    *num = (*num << 8) | data[GST_BUFFER_SIZE (buf) - size];
-    size--;
+  while (n < size) {
+    *num = (*num << 8) | data[n++];
   }
 
   /* make signed */
-  *num -= (1LL << ((8 * GST_BUFFER_SIZE (buf)) - 1));
+  if (negative) {
+    *num = *num - (1LL << ((8 * size) - 1));
+  }
 
   gst_buffer_unref (buf);
 
@@ -512,16 +516,16 @@ gst_ebml_read_float (GstEbmlRead *ebml,
   size = GST_BUFFER_SIZE (buf);
 
   if (size != 4 && size != 8 && size != 10) {
-    gst_element_error (GST_ELEMENT (ebml),
-		       "Invalid float element size %d at position %llu (0x%llx)",
-		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf));
+    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
+		       ("Invalid float element size %d at position %llu (0x%llx)",
+		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
     gst_buffer_unref (buf);
     return FALSE;
   }
 
   if (size == 10) {
-    gst_element_error (GST_ELEMENT (ebml),
-		       "FIXME! 10-byte floats unimplemented");
+    GST_ELEMENT_ERROR (ebml, CORE, NOT_IMPLEMENTED, (NULL),
+		       ("FIXME! 10-byte floats unimplemented"));
     gst_buffer_unref (buf);
     return FALSE;
   }
@@ -681,7 +685,7 @@ gst_ebml_read_header (GstEbmlRead *ebml,
   if (!(id = gst_ebml_peek_id (ebml, &level_up)))
     return FALSE;
   if (level_up != 0 || id != GST_EBML_ID_HEADER) {
-    gst_element_error (GST_ELEMENT (ebml), "Not a EBML file");
+    GST_ELEMENT_ERROR (ebml, STREAM, WRONG_TYPE, (NULL), (NULL));
     return FALSE;
   }
   if (!gst_ebml_read_master (ebml, &id))
