@@ -35,7 +35,8 @@ enum {
 
 enum {
   ARG_0,
-  ARG_BYTESPERREAD
+  ARG_BYTESPERREAD,
+  ARG_SYNC
 };
 
 /* elementfactory information */
@@ -79,6 +80,7 @@ static void 	gst_silence_get_property (GObject      *object,
 					  GParamSpec   *pspec);
 static GstElementStateReturn
 		gst_silence_change_state (GstElement   *element);
+static void gst_silence_set_clock (GstElement *element, GstClock *clock);
 
 static GstData *gst_silence_get		 (GstPad       *pad);
 static GstPadLinkReturn
@@ -141,11 +143,25 @@ gst_silence_class_init (GstSilenceClass *klass)
     g_param_spec_ulong("bytes_per_read","Bytes per read",
 		       "Bytes per buffer in one cycle",
                        0,G_MAXULONG,0,G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SYNC,
+      g_param_spec_boolean ("sync", "Sync", "Synchronize to clock",
+        TRUE, G_PARAM_READWRITE));
 
   gobject_class->set_property = gst_silence_set_property;
   gobject_class->get_property = gst_silence_get_property;
 
   gstelement_class->change_state = gst_silence_change_state;
+  gstelement_class->set_clock = gst_silence_set_clock;
+}
+
+static void
+gst_silence_set_clock (GstElement *element, GstClock *clock)
+{   
+  GstSilence *silence;
+  
+  silence = GST_SILENCE (element);
+
+  gst_object_replace ((GstObject **)&silence->clock, (GstObject *)clock);
 }
 
 static void 
@@ -267,6 +283,9 @@ gst_silence_get (GstPad *pad)
   buf = gst_buffer_new_and_alloc (src->bytes_per_read);
   GST_BUFFER_OFFSET (buf) = src->offset;
   GST_BUFFER_TIMESTAMP (buf) = src->timestamp;
+  if (src->sync && src->clock) {
+    gst_element_wait (GST_ELEMENT (src), GST_BUFFER_TIMESTAMP (buf));
+  }
   GST_BUFFER_DURATION (buf) = tdiff;
   GST_BUFFER_SIZE (buf) = src->bytes_per_read;
 
@@ -296,6 +315,9 @@ gst_silence_set_property (GObject      *object,
     case ARG_BYTESPERREAD:
       src->bytes_per_read = g_value_get_ulong (value);
       break;
+    case ARG_SYNC:
+      src->sync = g_value_get_boolean (value);
+      break;
     default:
       break;
   }
@@ -318,6 +340,9 @@ gst_silence_get_property (GObject    *object,
     case ARG_BYTESPERREAD:
       g_value_set_ulong (value, src->bytes_per_read);
       break;
+    case ARG_SYNC:
+      g_value_set_boolean (value, src->sync);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
