@@ -20,6 +20,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -28,6 +32,7 @@
 #include <unistd.h>
 /*#include "osdtools.h"*/
 /*#include "speedy.h"*/
+#include <glib.h>
 #include "vbiscreen.h"
 #include "gstvbidec.h"
 
@@ -382,6 +387,11 @@ void scroll_screen( vbiscreen_t *vs )
     vs->scroll = 26;
 }
 
+void vbiscreen_set_verbose( vbiscreen_t *vs, int verbose )
+{
+    vs->verbose = verbose;
+}
+
 void vbiscreen_new_caption( vbiscreen_t *vs, int indent, int ital, 
                             unsigned int colour, int row )
 {
@@ -497,7 +507,7 @@ void vbiscreen_set_current_cell( vbiscreen_t *vs, char text )
     int base;
     if( !vs ) return;
     base = ( ( vs->top_of_screen + vs->cury ) % ( 2 * ROWS ) ) * COLS;
-    if( isprint( text ) ) 
+    if( g_ascii_isprint( text ) ) 
         vs->text[ base + vs->curx + vs->indent ] = text;
     else
         vs->text[ base + vs->curx + vs->indent ] = ' ';
