@@ -48,26 +48,20 @@ enum {
   ARG_CONTROLPOINT
 };
 
-GST_PAD_TEMPLATE_FACTORY (src_factory,
+static GstStaticPadTemplate gst_volenv_src_template =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  gst_caps_new (
-    "volenv_src_int",
-    "audio/x-raw-int",
-      GST_AUDIO_INT_PAD_TEMPLATE_PROPS
-  )
+  GST_STATIC_CAPS ( GST_AUDIO_INT_PAD_TEMPLATE_CAPS )
 );
 
-GST_PAD_TEMPLATE_FACTORY (sink_factory,
+static GstStaticPadTemplate gst_volenv_sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  gst_caps_new (
-    "volenv_src_int",
-    "audio/x-raw-int",
-      GST_AUDIO_INT_PAD_TEMPLATE_PROPS
-  )
+  GST_STATIC_CAPS ( GST_AUDIO_INT_PAD_TEMPLATE_CAPS )
 );
 
 static void		gst_volenv_base_init		(gpointer g_class);
@@ -91,7 +85,7 @@ static GstElementClass *parent_class = NULL;
 /*static guint gst_filter_signals[LAST_SIGNAL] = { 0 }; */
 
 static GstPadLinkReturn
-volenv_connect (GstPad *pad, GstCaps *caps)
+volenv_link (GstPad *pad, const GstCaps *caps)
 {
   GstVolEnv *filter;
   GstPad *otherpad;
@@ -99,9 +93,7 @@ volenv_connect (GstPad *pad, GstCaps *caps)
   filter = GST_VOLENV (gst_pad_get_parent (pad));
   otherpad = (pad == filter->srcpad ? filter->sinkpad : filter->srcpad);
   
-  if (GST_CAPS_IS_FIXED (caps))
-    return gst_pad_try_set_caps (otherpad, caps);
-  return GST_PAD_LINK_DELAYED;
+  return gst_pad_try_set_caps (otherpad, caps);
 }
 
 GType
@@ -133,9 +125,9 @@ gst_volenv_base_init (gpointer g_class)
   gst_element_class_set_details (element_class, &volenv_details);
   
   gst_element_class_add_pad_template (element_class,
-	GST_PAD_TEMPLATE_GET (src_factory));
+      gst_static_pad_template_get (&gst_volenv_src_template));
   gst_element_class_add_pad_template (element_class,
-	GST_PAD_TEMPLATE_GET (sink_factory));
+      gst_static_pad_template_get (&gst_volenv_sink_template));
 }
 static void
 gst_volenv_class_init (GstVolEnvClass *klass)
@@ -166,9 +158,9 @@ static void
 gst_volenv_init (GstVolEnv *filter)
 {
   filter->sinkpad = gst_pad_new_from_template(
-	GST_PAD_TEMPLATE_GET (sink_factory), "sink");
+      gst_static_pad_template_get (&gst_volenv_sink_template), "sink");
   filter->srcpad = gst_pad_new_from_template(
-	GST_PAD_TEMPLATE_GET (src_factory), "src");
+      gst_static_pad_template_get (&gst_volenv_src_template), "src");
 
   filter->run_time = 0.0;
   filter->level = 1.0;
@@ -180,9 +172,9 @@ gst_volenv_init (GstVolEnv *filter)
 
   gst_element_add_pad(GST_ELEMENT(filter),filter->sinkpad);
   gst_pad_set_chain_function(filter->sinkpad,gst_volenv_chain);
-  gst_pad_set_link_function(filter->sinkpad, volenv_connect);
+  gst_pad_set_link_function(filter->sinkpad, volenv_link);
   gst_element_add_pad(GST_ELEMENT(filter),filter->srcpad);
-  gst_pad_set_link_function(filter->srcpad, volenv_connect);
+  gst_pad_set_link_function(filter->srcpad, volenv_link);
 }
 
 static void
@@ -196,8 +188,8 @@ gst_volenv_chain (GstPad *pad, GstData *_data)
   gint width;
   gint num_samples;
   gint sample_rate;
-
-  GstCaps *caps;
+  const GstCaps *caps = NULL;
+  GstStructure *structure;
 
   g_return_if_fail(pad != NULL);
   g_return_if_fail(GST_IS_PAD(pad));
@@ -211,13 +203,16 @@ gst_volenv_chain (GstPad *pad, GstData *_data)
 
   caps = GST_PAD_CAPS(pad);
   if (caps == NULL) {
-    gst_element_error (GST_ELEMENT (filter), "capsnego was never performed, bailing...\n");
+    GST_ELEMENT_ERROR (filter, CORE, NEGOTIATION, (NULL),
+                       ("format wasn't negotiated before chain function"));
     gst_buffer_unref (buf);
     return;
   }
 
-  gst_caps_get_int(caps, "width", &width);
-  gst_caps_get_int(caps, "rate", &sample_rate);
+  structure = gst_caps_get_structure (caps, 0);
+
+  gst_structure_get_int(structure, "width", &width);
+  gst_structure_get_int(structure, "rate", &sample_rate);
 
   in_data = (gint16 *)GST_BUFFER_DATA(buf);
   outbuf=gst_buffer_new();
@@ -259,7 +254,8 @@ gst_volenv_set_property (GObject *object, guint prop_id, const GValue *value, GP
 {
   gint rate = 0;
   GstVolEnv *filter;
-  GstCaps *caps;
+  const GstCaps *caps = NULL;
+  GstStructure *structure;
 
   /* it's not null if we got it, but it might not be ours */
   g_return_if_fail(GST_IS_VOLENV(object));
@@ -283,7 +279,8 @@ gst_volenv_set_property (GObject *object, guint prop_id, const GValue *value, GP
         }
         else 
         {
-          gst_caps_get_int (caps, "rate", &rate);
+	  structure = gst_caps_get_structure (caps, 0);
+          gst_structure_get_int (structure, "rate", &rate);
         }
     filter->arg_rise = g_value_get_double (value);
     filter->increase = filter->arg_rise / (double) rate;
